@@ -149,27 +149,27 @@ sub visitTypeDeclarator {
 		warn __PACKAGE__,"::visitTypeDecalarator $node->{idf} : empty array_size.\n"
 				unless (@{$node->{array_size}});
 
-		my $deref = "";
+		my $start = "";
 		my $nb;
 		my $first = 1;
 		foreach (@{$node->{array_size}}) {
-			$deref .= "*" unless ($first);
+			$start .= "[0]";
 			$nb .= " * " unless ($first);
 			$nb .= $_->{c_literal};
 			$first = 0;
 		}
 		print $FH "#define ADD_SIZE_",$node->{c_name},"(size,v) {\\\n";
 		print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-		print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"(v);\\\n";
-		print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"(v) + (",$nb,");\\\n";
+		print $FH "\t\tfor (",$node->{c_name},"_ptr = &(v)" . $start . ";\\\n";
+		print $FH "\t\t     ",$node->{c_name},"_ptr < &(v)" . $start . " + (",$nb,");\\\n";
 		print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 		print $FH "\t\t\tADD_SIZE_",$type->{c_name},"(size,*",$node->{c_name},"_ptr);\\\n";
 		print $FH "\t\t}\\\n";
 		print $FH "\t}\n";
 		print $FH "#define PUT_",$node->{c_name},"(ptr,v) {\\\n";
 		print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-		print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"(v);\\\n";
-		print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"(v) + (",$nb,");\\\n";
+		print $FH "\t\tfor (",$node->{c_name},"_ptr = &(v)" . $start . ";\\\n";
+		print $FH "\t\t     ",$node->{c_name},"_ptr < &(v)" . $start . " + (",$nb,");\\\n";
 		print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 		print $FH "\t\t\tPUT_",$type->{c_name},"(ptr,*",$node->{c_name},"_ptr);\\\n";
 		print $FH "\t\t}\\\n";
@@ -178,16 +178,16 @@ sub visitTypeDeclarator {
 			if (exists $self->{client}) {
 				print $FH "#define GET_inout_",$node->{c_name},"(ptr,v) {\\\n";
 				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
-				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
+				print $FH "\t\tfor (",$node->{c_name},"_ptr = &(*(v))" . $start . ";\\\n";
+				print $FH "\t\t     ",$node->{c_name},"_ptr < &(*(v))" . $start . " + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 				print $FH "\t\t\tGET_inout_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 				print $FH "#define GET_out_",$node->{c_name},"(ptr,v) {\\\n";
 				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
-				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
+				print $FH "\t\tfor (",$node->{c_name},"_ptr = &(*(v))" . $start . ";\\\n";
+				print $FH "\t\t     ",$node->{c_name},"_ptr < &(*(v))" . $start . " + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 				print $FH "\t\t\tGET_out_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
@@ -196,8 +196,8 @@ sub visitTypeDeclarator {
 				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 				print $FH "\t\tptr = ",$type->{c_name},"__alloc(",$nb,");\\\n";
 				print $FH "\t\tif (NULL == ptr) goto err;\\\n";
-				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
-				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
+				print $FH "\t\tfor (",$node->{c_name},"_ptr = &(*(v))" . $start . ";\\\n";
+				print $FH "\t\t     ",$node->{c_name},"_ptr < &(*(v))" . $start . " + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 				print $FH "\t\t\tGET_out_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
@@ -205,32 +205,32 @@ sub visitTypeDeclarator {
 			} else {
 				print $FH "#define GET_",$node->{c_name},"(ptr,v) {\\\n";
 				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
-				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
+				print $FH "\t\tfor (",$node->{c_name},"_ptr = &(*(v))" . $start . ";\\\n";
+				print $FH "\t\t     ",$node->{c_name},"_ptr < &(*(v))" . $start . " + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 				print $FH "\t\t\tGET_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 				print $FH "#define FREE_in_",$node->{c_name},"(v) {\\\n";
 				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
-				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
+				print $FH "\t\tfor (",$node->{c_name},"_ptr = &(*(v))" . $start . ";\\\n";
+				print $FH "\t\t     ",$node->{c_name},"_ptr < &(*(v))" . $start . " + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 				print $FH "\t\t\tFREE_in_",$type->{c_name},"(",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 				print $FH "#define FREE_inout_",$node->{c_name},"(v) {\\\n";
 				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
-				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
+				print $FH "\t\tfor (",$node->{c_name},"_ptr = &(*(v))" . $start . ";\\\n";
+				print $FH "\t\t     ",$node->{c_name},"_ptr < &(*(v))" . $start . " + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 				print $FH "\t\t\tFREE_inout_",$type->{c_name},"(",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 				print $FH "#define FREE_out_",$node->{c_name},"(v) {\\\n";
 				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
-				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
+				print $FH "\t\tfor (",$node->{c_name},"_ptr = &(*(v))" . $start . ";\\\n";
+				print $FH "\t\t     ",$node->{c_name},"_ptr < &(*(v))" . $start . " + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 				print $FH "\t\t\tFREE_out_",$type->{c_name},"(",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
@@ -238,8 +238,8 @@ sub visitTypeDeclarator {
 			}
 			print $FH "#define FREE_",$node->{c_name},"(v) {\\\n";
 			print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-			print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
-			print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
+			print $FH "\t\tfor (",$node->{c_name},"_ptr = &(*(v))" . $start . ";\\\n";
+			print $FH "\t\t     ",$node->{c_name},"_ptr < &(*(v))" . $start . " + (",$nb,");\\\n";
 			print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 			print $FH "\t\t\tFREE_",$type->{c_name},"(",$node->{c_name},"_ptr);\\\n";
 			print $FH "\t\t}\\\n";
@@ -247,8 +247,8 @@ sub visitTypeDeclarator {
 		} else {
 			print $FH "#define GET_",$node->{c_name},"(ptr,v) {\\\n";
 			print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-			print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
-			print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
+			print $FH "\t\tfor (",$node->{c_name},"_ptr = &(*(v))" . $start . ";\\\n";
+			print $FH "\t\t     ",$node->{c_name},"_ptr < &(*(v))" . $start . " + (",$nb,");\\\n";
 			print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
 			print $FH "\t\t\tGET_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 			print $FH "\t\t}\\\n";
@@ -374,11 +374,11 @@ sub visitArray {
 	my $self = shift;
 	my($node) = @_;
 
-	my $deref = "";
+	my $start = "";
 	my $nb;
 	my $first = 1;
 	foreach (@{$node->{array_size}}) {
-		$deref .= "*" unless ($first);
+		$start .= "[0]";
 		$nb .= " * " unless ($first);
 		$nb .= $_->{c_literal};
 		$first = 0;
@@ -387,48 +387,48 @@ sub visitArray {
 	my $type = $self->_get_defn($node->{type});
 	$self->{add_size}  .= "\t\t{\\\n";
 	$self->{add_size}  .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
-	$self->{add_size}  .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)." . $self->{union} . $node->{c_name} . ");\\\n";
-	$self->{add_size}  .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)." . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
+	$self->{add_size}  .= "\t\t\tfor (" . $node->{c_name} . "_ptr = &((v)." . $self->{union} . $node->{c_name} . ")" . $start . ";\\\n";
+	$self->{add_size}  .= "\t\t\t     " . $node->{c_name} . "_ptr < &((v)." . $self->{union} . $node->{c_name} . ")" . $start . " + (" . $nb . ");\\\n";
 	$self->{add_size}  .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
 	$self->{add_size}  .= "\t\t\t\tADD_SIZE_" . $type->{c_name} . "(size,*" . $node->{c_name} . "_ptr);\\\n";
 	$self->{add_size}  .= "\t\t\t}\\\n";
 	$self->{add_size}  .= "\t\t}\\\n";
 	$self->{put}       .= "\t\t{\\\n";
 	$self->{put}       .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
-	$self->{put}       .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)." . $self->{union} . $node->{c_name} . ");\\\n";
-	$self->{put}       .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)." . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
+	$self->{put}       .= "\t\t\tfor (" . $node->{c_name} . "_ptr = &((v)." . $self->{union} . $node->{c_name} . ")" . $start . ";\\\n";
+	$self->{put}       .= "\t\t\t     " . $node->{c_name} . "_ptr < &((v)." . $self->{union} . $node->{c_name} . ")" . $start . " + (" . $nb . ");\\\n";
 	$self->{put}       .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
 	$self->{put}       .= "\t\t\t\tPUT_" . $type->{c_name} . "(ptr,*" . $node->{c_name} . "_ptr);\\\n";
 	$self->{put}       .= "\t\t\t}\\\n";
 	$self->{put}       .= "\t\t}\\\n";
 	$self->{get}       .= "\t\t{\\\n";
 	$self->{get}       .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
-	$self->{get}       .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
-	$self->{get}       .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
+	$self->{get}       .= "\t\t\tfor (" . $node->{c_name} . "_ptr = &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . ";\\\n";
+	$self->{get}       .= "\t\t\t     " . $node->{c_name} . "_ptr < &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . " + (" . $nb . ");\\\n";
 	$self->{get}       .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
 	$self->{get}       .= "\t\t\t\tGET_" . $type->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
 	$self->{get}       .= "\t\t\t}\\\n";
 	$self->{get}       .= "\t\t}\\\n";
 	$self->{get_in}    .= "\t\t{\\\n";
 	$self->{get_in}    .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
-	$self->{get_in}    .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
-	$self->{get_in}    .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
+	$self->{get_in}    .= "\t\t\tfor (" . $node->{c_name} . "_ptr = &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . ";\\\n";
+	$self->{get_in}    .= "\t\t\t     " . $node->{c_name} . "_ptr < &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . " + (" . $nb . ");\\\n";
 	$self->{get_in}    .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
 	$self->{get_in}    .= "\t\t\t\tGET_in_" . $type->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
 	$self->{get_in}    .= "\t\t\t}\\\n";
 	$self->{get_in}    .= "\t\t}\\\n";
 	$self->{get_inout} .= "\t\t{\\\n";
 	$self->{get_inout} .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
-	$self->{get_inout} .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
-	$self->{get_inout} .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
+	$self->{get_inout} .= "\t\t\tfor (" . $node->{c_name} . "_ptr = &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . ";\\\n";
+	$self->{get_inout} .= "\t\t\t     " . $node->{c_name} . "_ptr < &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . " + (" . $nb . ");\\\n";
 	$self->{get_inout} .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
 	$self->{get_inout} .= "\t\t\t\tGET_inout_" . $type->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
 	$self->{get_inout} .= "\t\t\t}\\\n";
 	$self->{get_inout} .= "\t\t}\\\n";
 	$self->{get_out}   .= "\t\t{\\\n";
 	$self->{get_out}   .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
-	$self->{get_out}   .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
-	$self->{get_out}   .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
+	$self->{get_out}   .= "\t\t\tfor (" . $node->{c_name} . "_ptr = &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . ";\\\n";
+	$self->{get_out}   .= "\t\t\t     " . $node->{c_name} . "_ptr < &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . " + (" . $nb . ");\\\n";
 	$self->{get_out}   .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
 	$self->{get_out}   .= "\t\t\t\tGET_" . $type->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
 	$self->{get_out}   .= "\t\t\t}\\\n";
@@ -436,8 +436,8 @@ sub visitArray {
 	if (defined $type->{length}) {
 		$self->{free}      .= "\t\t{\\\n";
 		$self->{free}      .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
-		$self->{free}      .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
-		$self->{free}      .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
+		$self->{free}      .= "\t\t\tfor (" . $node->{c_name} . "_ptr = &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . ";\\\n";
+		$self->{free}      .= "\t\t\t     " . $node->{c_name} . "_ptr < &((v)->" . $self->{union} . $node->{c_name} . ")" . $start . " + (" . $nb . ");\\\n";
 		$self->{free}      .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
 		$self->{free}      .= "\t\t\t\tFREE_" . $type->{c_name} . "(" . $node->{c_name} . "_ptr);\\\n";
 		$self->{free}      .= "\t\t\t}\\\n";
